import "Turbine";
import "Turbine.Debug";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "DhorPlugins.Extensions";
import "DhorPlugins.Utils.BitOps";


--[[ This is the main window for the Champ Fervour ]]--
--[[ It handles the loading and saving the settings,	                      ]]--
--[[ creating the lists of travel shortcuts, creates	                      ]]--
--[[ and handles the context menu.						                         ]]--


MainWindow = class( Turbine.UI.Extensions.SimpleWindow );

function MainWindow:Constructor()
	Turbine.UI.Extensions.SimpleWindow.Constructor( self );	
	
   -- get the player class and race
   player = Turbine.Gameplay.LocalPlayer.GetInstance();
   playerClass = player:GetClass();
   
   -- quit if the class is not a champion
   if(playerClass ~= Turbine.Gameplay.Class.Champion) then
      Turbine.Shell.WriteLine("This plugin will only work for champions.");
      return;
   end
   
   -- create the settings	
	settings = { };
   settingStrings = { }; 	
	self.minWidth = 21;
	self.minHeight = 21;
	self.options = nil;
	
	-- load the player saved settings
	self:LoadSettings();

	-- configure the main window
	self:SetPosition(settings.positionX,settings.positionY);	
	self:SetSize( settings.width, settings.height );
	self:SetOpacity( settings.mainMinOpacity );
	self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self:SetZOrder(1);
	self:SetBackColor(settings.backColor);
	self:SetVisible(true); 
   
   -- create a control for resizing the window
   self.sizeControl = Turbine.UI.Control();
   self.sizeControl:SetParent(self);
   self.sizeControl:SetZOrder(100);
   self.sizeControl:SetSize(20,20);
   self.sizeControl:SetBackColorBlendMode(Turbine.UI.BlendMode.Screen)
   self.sizeControl:SetBackColor(Turbine.UI.Color(0.01,1,1,1));
   self.sizeControl:SetPosition(self:GetWidth() - self.sizeControl:GetWidth(),
                            self:GetHeight() - self.sizeControl:GetHeight());   
   
   -- add the labels for the fervour level   
   self.labels = { };
   for i = 1,5,1 do
      self.labels[i] = Turbine.UI.Label();		
		self.labels[i]:SetParent( self );
		self.labels[i]:SetBackColor(settings.fervourColor);
		self.labels[i]:SetZOrder(50);
		self.labels[i]:SetMouseVisible(false);
   end
   
   -- add the label for the fervour number
   
   self.numberLabel = Turbine.UI.Label();		
	self.numberLabel:SetParent( self );
	self.numberLabel:SetSize(19,19);
   self.numberLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro19);	
   self.numberLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.numberLabel:SetZOrder(51);
	self.numberLabel:SetMouseVisible(false);  
   
	-- create a single context menu to use on all panels
	menu = Turbine.UI.ContextMenu(self);
   self:SetupMenu();
		
	-- champ attribute
	self.champFervour = player:GetClassAttributes();
	self.fervourLogging = false;
	self.loggingStartTime = nil;
	self.lastCombatLog = { };
		
   -- timers for events
	self.lastTime = 0;
   self.lastMove = 0;
   
   -- track the hidden state of the UI
   self.hidden = false;
   
   -- setup the labels for the fevour
   self:UpdateFervour(self.champFervour:GetFervor()); 
   self:UpdateLabels();
   self:UpdateColors(self.champFervour:GetStance());
      
   
--[[ Event Handlers ]]--

	-- make sure we listen for updates
	self:SetWantsUpdates(true);
	self:SetWantsKeyEvents(true);
	
   local isMoving = false;
	local x = 0;
	local y = 0;

   -- handle window mouse down event (not quickslot or button events)
	self.MouseDown = function(sender, args )
	   -- if its the left button, prepare to move the window by setting
	   -- the flag, and storing the current mouse position
		if (args.Button == Turbine.UI.MouseButton.Left  and settings.lock == 0 and settings.skin == 0) then
			isMoving = true;
			x = args.X;
			y = args.Y;
		elseif(args.Button == Turbine.UI.MouseButton.Right and player:IsInCombat() == false) then
		   menu:ShowMenu();
		end
	end

   -- handle window mouse up event
	self.MouseUp = function(sender, args )
	   -- check if its the left mouse button, otherwise ignore
		if (args.Button == Turbine.UI.MouseButton.Left) then
		
		   -- clear the flag
			isMoving = false;

         -- save the current position settings
			local one,two = self:GetPosition();
			settings.positionX = one;
			settings.positionY = two;
			self:SaveSettings();
		end
	end

   -- handle window mouse move event
	self.MouseMove = function(sender, args )
	   -- if the moving flag is set, move the window
		if (isMoving) then
		   -- get the old position of the window
			local oldX,oldY = self:GetPosition();
			
			-- update the window position based on the previous mouse location,
			-- previous window position, and the current mouse position
			self:SetPosition(oldX + args.X - x,oldY + args.Y - y);	
		end
	end
   	
	-- check if our position has changed, and save the settings if so
	self.PositionChanged = function(sender, args )
		one,two = self:GetPosition();
		settings.positionX = one;
		settings.positionY = two;
		self:SaveSettings();
	end
	
   -- manage hiding the UI  with F12
   self.KeyDown = function( sender, args)
      if(args.Action == 268435635) then
         if(self.hidden == false) then
             self.currentVisState = self:IsVisible();
             self.hidden = true;
             self:SetVisible(false);             
         else
             self.hidden = false;
             self:SetVisible(self.currentVisState);
         end
      end
   end
   
   -- check if the player's fervour level changes
   self.champFervour.FervorChanged = function(sender, args)
           
      if(self.fervourLogging == true) then
         table.insert(self.lastCombatLog,{Turbine.Engine.GetGameTime() - self.loggingStartTime,self.champFervour:GetFervor()});
      end
      self:UpdateColors(self.champFervour:GetStance());
      self:UpdateFervour(self.champFervour:GetFervor());
   end
   
   -- watch if the player's stance changes
   self.champFervour.StanceChanged = function(sender, args)
      self:UpdateColors(self.champFervour:GetStance());
   end
   
   -- check if the combat state of the player changed, to adjust the opacity
   -- of the window
   
   incombatChangedHandler = function(sender, args)

      if(player:IsInCombat()) then
           self:SetOpacity(settings.mainMaxOpacity)
           self.fervourLogging = true;
	        self.loggingStartTime = Turbine.Engine.GetGameTime();
	        self.lastCombatLog = { };
	        
	        table.insert(self.lastCombatLog,{0,self.champFervour:GetFervor()});
      else
         self.fervourLogging = false;
	      table.insert(self.lastCombatLog,{Turbine.Engine.GetGameTime() - self.loggingStartTime,self.champFervour:GetFervor()});
         self:SetOpacity(settings.mainMinOpacity)
	   end
   end    
   AddCallback(player, "InCombatChanged", incombatChangedHandler);

	-- change the background color of the resize button on entry to help
	-- show exactly where the button is
	self.sizeControl.MouseEnter = function(sender,args)
     if(settings.lock == 0 and settings.skin == 0) then     	  
      self.sizeControl:SetBackColor(Turbine.UI.Color(0.9,1,1,1));
     end
	end
	
	-- return background color to normal on mouse leave
	self.sizeControl.MouseLeave = function(sender,args)
	   self.sizeControl:SetBackColor(Turbine.UI.Color(0.01,1,1,1));
	end
	
	-- check if someone is going to start dragging the resize control
	self.sizeControl.MouseDown = function(sender,args)
	
	  -- check if resize has been disabled
	  if(self.disableResize or settings.lock == 1 or settings.skin == 1) then
	     return;
	  end  
	
	  -- set start location of resize and enable dragging
	  sender.dragStartX = args.X;
	  sender.dragStartY = args.Y;
	  sender.dragging = true;
	end
	
	self.sizeControl.MouseUp = function(sender,args)
	  -- disable dragging, then update settings
	  sender.dragging = false;	  
	  
	  settings.width = self:GetWidth();
	  settings.height = self:GetHeight();	  
	  self:SaveSettings();
	end
	
	-- perform the resize
	self.sizeControl.MouseMove = function(sender,args)	
	  	 local width, height = self:GetSize();
      
         -- if resize dragging, then resize the window 		
		 if ( sender.dragging ) then
			   self:SetSize( width + ( args.X - sender.dragStartX ), 
                        height + ( args.Y - sender.dragStartY ) );
		 end
		 
		 -- update the size of the window
		 self:UpdateSize();      
	end
   
   Plugins["ChampionFervour"].Load = function(sender,args)
      Turbine.Shell.WriteLine("Champion Fervour version " .. Plugins["ChampionFervour"]:GetVersion() .. " loaded");     
      Turbine.Shell.WriteLine("Note: Due to a bug in the stance detection, the only color that will show is the default fervour color.");      
      Turbine.Shell.WriteLine("Once Turbine fixes the bug, then the other color settings will be enabled.");
   end                          
end

function MainWindow:UpdateSize()
         -- check that the window is not smaller than min width	
		 if (self:GetWidth() < self.minWidth) then
		       self:SetWidth(self.minWidth);
		 end
		
		 -- check that the window is not smaller than min height
		 if (self:GetHeight() < self.minHeight) then
		       self:SetHeight(self.minHeight);
       end
      
       -- set the new location of the resize control
		 self.sizeControl:SetPosition( 
                           self:GetWidth() - self.sizeControl:GetWidth(), 
                           self:GetHeight() - self.sizeControl:GetHeight() );
      
                           
       self:SaveSettings();
       self:UpdateLabels();
              
end

function MainWindow:SetupMenu()
	self.options = Turbine.UI.MenuItem(optionsString);
	self.fevourLog = Turbine.UI.MenuItem(logString);
	menuItems = menu:GetItems();
	menuItems:Add(self.options);
	menuItems:Add(self.fevourLog);
	
	-- open the option window when clicked
	menuItems:Get(1).Click = function(sender, args)
	   self:OpenOptions();
	end
	
	menuItems:Get(2).Click = function(sender,args)
	   self:ShowLog();
	end
end  

function MainWindow:LoadSettings()
	-- load the self.settings.  If a value is not available, set a default value
	settingStrings = Turbine.PluginData.Load( Turbine.DataScope.Character, 
                                                "champFervourSettings" );

	if ( type( settingStrings ) ~= "table" ) then
		settingStrings = { };
	end
	
	if ( not settingStrings.lock or settingStrings.lock == "nil") then
	   settingStrings.lock = tostring(0);
	end
	
	if ( not settingStrings.width or settingStrings.width == "nil") then
	   settingStrings.width = tostring(200);
	end
	
	if (not settingStrings.height or settingStrings.height == "nil") then
	   settingStrings.height = tostring(50);
	end

	if ( not settingStrings.positionX or settingStrings.positionX == "nil" ) then
		settingStrings.positionX = tostring(200); 
	end

	if ( not settingStrings.positionY or settingStrings.positionY == "nil" ) then
		settingStrings.positionY = tostring(200);
	end
	
	if ( not settingStrings.hideOnStart or settingStrings.hideOnStart == "nil" ) then
	   settingStrings.hideOnStart = tostring(0);
	end
   	
	if ( not settingStrings.hideOnCombat or settingStrings.hideOnCombat == "nil" ) then
	   settingStrings.hideOnCombat = tostring(0);
	end
	
	if ( not settingStrings.horizontal or settingStrings.horizontal == "nil" ) then
	   settingStrings.horizontal = tostring(1);
	end

	if ( not settingStrings.mainMaxOpacity or settingStrings.mainMaxOpacity == "nil" ) then
	   settingStrings.mainMaxOpacity = tostring(1);
	end
	
	if ( not settingStrings.mainMinOpacity or settingStrings.mainMinOpacity == "nil" ) then
	  settingStrings.mainMinOpacity = tostring(0.5);
	end
	
	if ( not settingStrings.showBars or settingStrings.showBars == "nil" ) then
	  settingStrings.showBars = tostring(1);
	end
	
	if ( not settingStrings.showNumber or settingStrings.showNumber == "nil" ) then
	  settingStrings.showNumber = tostring(1);
	end
	
	if ( not settingStrings.skin or settingStrings.skin == "nil") then
	   settingStrings.skin = tostring(0);
	end
   	
	if ( not settingStrings.fervourColor or settingStrings.fervourColor == "nil" ) then
	  settingStrings.fervourColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.fervourAColor or settingStrings.fervourAColor == "nil" ) then
	  settingStrings.fervourAColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.fervourFColor or settingStrings.fervourFColor == "nil" ) then
	  settingStrings.fervourFColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.fervourGColor or settingStrings.fervourGColor == "nil" ) then
	  settingStrings.fervourGColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.backColor or settingStrings.backColor == "nil" ) then
	  settingStrings.backColor = Turbine.UI.Color(1,1,1,1);
	end
	
	if ( not settingStrings.numberColor or settingStrings.numberColor == "nil" ) then
	  settingStrings.numberColor = Turbine.UI.Color(1,1,1,1);
	end
	
	settings = { };
	

	-- convert from strings if necessary 
   for i,v in pairs(settingStrings) do
      
      if(type(v) == "string") then
         settings[i] = tonumber(v);
      elseif(type(v) == "table") then
         self:convertColorToNumbers(i,v);
      else
         settings[i] = v;
      end
   end
end

function MainWindow:SaveSettings()
	-- save the settings
	
	settingStrings = { };
	
	for i,v in pairs(settings) do
	   if(type(v) ~= "table") then
	      settingStrings[i] = tostring(v);
	   else
	      self:convertColorToStrings(i,v);
	   end
	end
	
	Turbine.PluginData.Save( Turbine.DataScope.Character,
                                  "champFervourSettings", settingStrings );
end

function MainWindow:convertColorToStrings(index, value)

   tempTable = { };

   for i,v in pairs(value) do
      if(type(v) == "number") then
         tempTable[i] = tostring(v);
      else
         tempTable[i] = v;
      end  
   end

   settingStrings[index] = tempTable;

end

function MainWindow:convertColorToNumbers(index, value)

   tempColor = Turbine.UI.Color(0,0,0,0);

   for i,v in pairs(value) do   
     if(type(v) == "string") then
        tempColor[i] = tonumber(v);
     else
        tempColor[i] = v;
     end
   end

   settings[index] = tempColor;

end

function MainWindow:UpdateLabels()

   if(settings.skin == 1) then
	   self:SetPosition(sMainWindowLocation[1],sMainWindowLocation[2]);	
	   self:SetSize( sMainWindowSize[1], sMainWindowSize[2] );
	   self:SetBackColor(Turbine.UI.Color(sMainWindowBackColorOpacity,sMainWindowBackColor[1],sMainWindowBackColor[2],sMainWindowBackColor[3]));
	   
      -- set the visibility and size of the number label
      self.numberLabel:SetVisible(settings.showNumber == 1);
      self.numberLabel:SetSize(sFNumberSize[1],sFNumberSize[2]);   
      self.numberLabel:SetPosition(sFNumberLocation[1],sFNumberLocation[2]);
	   self.numberLabel:SetBackColor(Turbine.UI.Color(sFNumberBackColorOpacity,sFNumberBackColorD[1],sFNumberBackColorD[2],sFNumberBackColorD[3]));
   else
      self:SetBackColor(settings.backColor); 
	   
      -- set the visibility and size of the number label
      self.numberLabel:SetVisible(settings.showNumber == 1);
      self.numberLabel:SetSize(19,19);   
      self.numberLabel:SetPosition(self:GetWidth() / 2 - 9,self:GetHeight() / 2 - 9);  
   end

   -- loop through the labels and adjust the size and position of each
   for i = 1,5,1 do
   
      if(settings.skin == 0) then
         local labelWidth = 0;
         local labelHeight = 0;
         local labelPosX = 0;
         local labelPosY = 0;
   
         if(settings.horizontal == 1) then
            labelWidth = (self:GetWidth() - 10)/5;
            labelHeight = self:GetHeight()-2;
            labelPosX = ((i-1) * labelWidth + i);
            labelPosY = 1;    
            --self.labels[i]:SetSize(labelWidth,labelHeight);
            --self.labels[i]:SetPosition((i-1) * labelWidth + i, 1);
         else
            labelHeight = (self:GetHeight() - 10)/5;
            labelWidth = self:GetWidth() - 2;
            labelPosX = 1;
            labelPosY = self:GetHeight() - ((i-1) * labelHeight + i) - labelHeight;    
            --self.labels[i]:SetSize(self:GetWidth() - 2,labelHeight);
            --self.labels[i]:SetPosition(1,self:GetHeight() - ((i-1) * labelHeight + i) - labelHeight);
         end
      
         self.labels[i]:SetSize(labelWidth,labelHeight);
         self.labels[i]:SetPosition(labelPosX,labelPosY); 
      else         
         self.labels[i]:SetSize(sFPipSizes[i][1],sFPipSizes[i][2]);
         self.labels[i]:SetPosition(sFPipLocations[i][1],sFPipLocations[i][2]);
      end 
   end                                                 
   
         
end

function MainWindow:UpdateFervour(level)

   -- loop through all the labels and set the visibility based on the
   -- current level of fervour 
     
   for i = 1,5,1 do
     if(i <= level and (settings.showBars == 1)) then
       self.labels[i]:SetVisible(true);
     else
       self.labels[i]:SetVisible(false);       
     end
   end   
   
   -- update the number label
   self.numberLabel:SetText(level); 
end

function MainWindow:UpdateColors(stance)

   -- default color
   colorToUse = Turbine.UI.Color(1,1,1,1);
   textBackColorToUse = Turbine.UI.Color(sFNumberBackColorOpacity,sFNumberBackColorD[1],sFNumberBackColorD[2],sFNumberBackColorD[3]);
   textColorToUse = Turbine.UI.Color(sFNumberTextColorOpactiy,sFNumberTextColorD[1],sFNumberTextColorD[2],sFNumberTextColorD[3]);
   pipBackColorArrayToUse = sFPipBackColorsD;
   pipArrayToUse = sFPipImageD;
   numberImageToUse = sFNumberImageD;
   
   -- select color based on stance
   if(stance == Turbine.Gameplay.Attributes.ChampionStance.None) then
      colorToUse = settings.fervourColor;
      pipArrayToUse = sFPipImageD;
      numberImageToUse = sFNumberImageD;
      pipBackColorArrayToUse = sFPipBackColorsD;
      textColorToUse = Turbine.UI.Color(sFNumberTextColorOpactiy,sFNumberTextColorD[1],sFNumberTextColorD[2],sFNumberTextColorD[3]);
      textBackColorToUse = Turbine.UI.Color(sFNumberBackColorOpacity,sFNumberBackColorD[1],sFNumberBackColorD[2],sFNumberBackColorD[3]);
      --Turbine.Shell.WriteLine("No Stance");        	
   elseif(stance == Turbine.Gameplay.Attributes.ChampionStance.Ardor) then
      colorToUse = settings.fervourAColor;
      pipArrayToUse = sFPipImageA;
      numberImageToUse = sFNumberImageA;
      pipBackColorArrayToUse = sFPipBackColorsA;
      textColorToUse = Turbine.UI.Color(sFNumberTextColorOpactiy,sFNumberTextColorA[1],sFNumberTextColorA[2],sFNumberTextColorA[3]);
      textBackColorToUse = Turbine.UI.Color(sFNumberBackColorOpacity,sFNumberBackColorA[1],sFNumberBackColorA[2],sFNumberBackColorA[3]);
      --Turbine.Shell.WriteLine("In Ardor");          	
   elseif(stance == Turbine.Gameplay.Attributes.ChampionStance.Fervor) then
      colorToUse = settings.fervourFColor;
      pipArrayToUse = sFPipImageF;
      numberImageToUse = sFNumberImageF;
      pipBackColorArrayToUse = sFPipBackColorsF;
      textColorToUse = Turbine.UI.Color(sFNumberTextColorOpactiy,sFNumberTextColorF[1],sFNumberTextColorF[2],sFNumberTextColorF[3]);
      textBackColorToUse = Turbine.UI.Color(sFNumberBackColorOpacity,sFNumberBackColorF[1],sFNumberBackColorF[2],sFNumberBackColorF[3]);
      --Turbine.Shell.WriteLine("In Fervor");          	
   elseif(stance == Turbine.Gameplay.Attributes.ChampionStance.Glory) then
      colorToUse = settings.fervourGColor;
      pipArrayToUse = sFPipImageG; 
      numberImageToUse = sFNumberImageG;
      pipBackColorArrayToUse = sFPipBackColorsG;
      textColorToUse = Turbine.UI.Color(sFNumberTextColorOpactiy,sFNumberTextColorG[1],sFNumberTextColorG[2],sFNumberTextColorG[3]);
      textBackColorToUse = Turbine.UI.Color(sFNumberBackColorOpacity,sFNumberBackColorG[1],sFNumberBackColorG[2],sFNumberBackColorG[3]);
      --Turbine.Shell.WriteLine("In Glory");          	
   end
   
   -- use defaults due to bug
   colorToUse = settings.fervourColor;
   pipArrayToUse = sFPipImageD;
   textColorToUse = Turbine.UI.Color(sFNumberTextColorOpactiy,sFNumberTextColorD[1],sFNumberTextColorD[2],sFNumberTextColorD[3]);
   textBackColorToUse = Turbine.UI.Color(sFNumberBackColorOpacity,sFNumberBackColorD[1],sFNumberBackColorD[2],sFNumberBackColorD[3]);
   numberImageToUse = sFNumberImageD;
   pipBackColorArrayToUse = sFPipBackColorsD;
      
   -- set the color to all the labels
   for i = 1,5,1 do
      if(settings.skin == 0) then
         self.labels[i]:SetBackColor(colorToUse);
         self.labels[i]:SetBackground(nil);
	      self.labels[i]:SetBlendMode(Turbine.UI.BlendMode.Undefined);
	      self.labels[i]:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
      else
         self.labels[i]:SetBackColor(Turbine.UI.Color(sFPipBackColorOpacity,pipBackColorArrayToUse[i][1],pipBackColorArrayToUse[i][2],pipBackColorArrayToUse[i][3]));
	      self.labels[i]:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	      if(pipArrayToUse[i] == nil or pipArrayToUse[i] == "nil") then
	        self.labels[i]:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	      else
           self.labels[i]:SetBackground(pipArrayToUse[i]);
	        self.labels[i]:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);
	      end
      end      
   end
       
   
   if(settings.skin == 1) then
      self.numberLabel:SetForeColor(textColorToUse);
      
      if(sMainWindowImage == nil or sMainWindowImage == "nil") then
	     self:SetBlendMode(Turbine.UI.BlendMode.Undefined);
	     self:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);	   
	   else
	     self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	     self:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);
	     self:SetBackground(sMainWindowImage);
	   end
	   
	   if(numberImageToUse == nil or numberImageToUse == "nil") then
	      self.numberLabel:SetBlendMode(Turbine.UI.BlendMode.Undefined);
	      self.numberLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);	   
	   else 	   
	      self.numberLabel:SetBackground(numberImageToUse);
	      self.numberLabel:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	      self.numberLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);
	   end
	else
      self.numberLabel:SetBackColor(colorToUse);  
      self.numberLabel:SetForeColor(settings.numberColor);
      
	   self:SetBlendMode(Turbine.UI.BlendMode.Undefined);
	   self:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	   
	   self.numberLabel:SetBlendMode(Turbine.UI.BlendMode.Undefined);
	   self.numberLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);	
	end
   
end   

-- for debug purposes
function MainWindow:DoDump()
	Turbine.Debug.Table.Dump(DhorPlugins.champFervour);
end

-- simple function to open the options window
function MainWindow:OpenOptions()
    self.options = DhorPlugins.ChampionFervour.OptionsWindow(self);
end

-- function to close the options window if it exists
function MainWindow:CloseOptions()
   if(self.options ~= nil) then
      self.options:SetVisible(false);
   end
   self.options = nil;
end

function MainWindow:UpdateOpacity()
   self:SetOpacity(settings.mainMinOpacity);
end

function MainWindow:LockWindow()
   Turbine.Shell.WriteLine(lockWindowString);
   settings.lock = 1;
   self:SaveSettings();
   
end 

function MainWindow:UnlockWindow()
   Turbine.Shell.WriteLine(unlockWindowString);
   settings.lock = 0;
   self:SaveSettings();
end

function MainWindow:ShowLog()
   self.logWindow = DhorPlugins.ChampionFervour.LogWindow(self,self.lastCombatLog);
end

-- function to close the options window if it exists
function MainWindow:CloseLog()
   if(self.logWindow ~= nil) then
      self.logWindow:SetVisible(false);
   end
   self.logWindow = nil;
end

function AddCallback(object, event, callback)
	if (object[event] == nil) then
        	object[event] = callback;
	else
        	if (type(object[event]) == "table") then
           		table.insert(object[event], callback);
	        else
        		object[event] = {object[event], callback};
	        end
	end
	return callback;
end